/*******************************************************************************
***  This file is part of Pecunia.                                           ***
***                                                                          ***
***  Copyright (C) 2017, 2018, 2019, 2020, 2021, 2023, 2024                  ***
***  John Schneiderman <Licencing _AT_ Schneiderman _DOT_ me>                ***
***                                                                          ***
***  This program is free software: you can redistribute it and/or modify it ***
***  under the terms of the GNU Lesser General Public License as published   ***
***  by the Free Software Foundation, either version 3 of the License, or    ***
***  (at your option) any later version.                                     ***
***                                                                          ***
***  This program is distributed in the hope that it will be useful, but     ***
***  WITHOUT ANY WARRANTY; without even the implied warranty of              ***
***  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    ***
***  See the GNU Lesser General Public License for more details.             ***
***                                                                          ***
***  You should have received a copy of the GNU Lesser General Public License***
***  along with this program. If not, see <http://www.gnu.org/licenses/>.    ***
*******************************************************************************/
#ifndef PECUNIA_CURRENCY_INTERNAL_ADJUSTMENTS_H_
#define PECUNIA_CURRENCY_INTERNAL_ADJUSTMENTS_H_

#include <cstdint>

#include <pecunia/MoneyTypes.h>


namespace pecunia
{

class FloatingPoint;

namespace currency
{

enum class Currency : std::uint16_t;
class Money;

namespace internal
{

MinorUnit fromMinorMinorTruncated (const MinorUnit minorAndSub, const uint8_t subMinorUnitDigits);
MinorUnit fromMinorSubMinor(const MinorUnit minorAndSub, const uint8_t subMinorUnitDigits);
MajorUnit fromFloatingPointMajor(
	const pecunia::FloatingPoint& value,
	const pecunia::currency::Currency& code,
	const std::uint8_t subMinorUnitDigits,
	const pecunia::FloatingPointRounderFunction& rounder
);
MinorUnit fromFloatingPointMinor(
	const pecunia::FloatingPoint& value,
	const pecunia::currency::Currency& code,
	const std::uint8_t subMinorUnitDigits,
	const pecunia::FloatingPointRounderFunction& rounder
);
MinorUnit fromFloatingPointSubMinor(
	const pecunia::FloatingPoint& value,
	const pecunia::currency::Currency& code,
	const std::uint8_t subMinorUnitDigits,
	const pecunia::FloatingPointRounderFunction& rounder
);
MajorUnit normaliseAmount(
	const pecunia::currency::Money& from,
	const pecunia::currency::Currency& to
);

}}}

#endif
